@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # coding: utf-8
 
-# In[2]:
+# In[ ]:
 
 
 get_ipython().run_line_magic('matplotlib', 'inline')
@@ -18,53 +18,7 @@ from scipy.sparse import diags
 
 class fock_space:
     def __init__(self, about_excitations, global_exc, statistics):
-        """The main elements anf functions of the Fock space:
-        
-        Parameters
-        ----------
-        about_excitations: int
-                            Number of modes
-        
-                        or array like
-                             Array of maximum numbers of local excitations on each site
-                           
-                            
-               global_exc: int
-                            Global limiNt on the total number of excitations
-                
-                statistics: 'Bose' or 'Fermi'
-                             Establishes the statistics of the Fock space. 
-                            
-        Returns
-        -------
-        1) self.modes: int
-            A number of modes
-            
-        2) self.local_exc: numpy.array
-            An array of maximum numbers of local excitations on each site  
-            
-        3) self.states_list: list of tuples
-            A list of all the possible states
-            
-        4) self.find_index: dict
-            A dictionary for translating a state tuple to a state index
-            
-        5) self.dimension : int
-            A number of all posible states
-            
-        6) self.emptyH: scipy sparse matrix
-            A sparse matrix for hamiltonian
-            
-        7) self.eye: scipy sparse matrix
-            An idenitiy operator
-        
-        8) self annihilate: scipy sparse matrix
-            A list of annihilation operators for each mode(from 0 to self.modes-1)
-            
-        9) self create: scipy sparse matrix
-            A list of creation operators for each mode(from 0 to self.modes-1)
-            
-        """
+
         self.global_exc = global_exc #internal parameter
         self.statistics = statistics #internal parameter
             
@@ -202,50 +156,17 @@ class fock_space:
                   
                 
     def sigmax(self, i):
-        """
-        A Pauli operator for i-th mode 
 
-        Parameters
-        ----------
-            i: int
-                An index of the mode                                          
-                            
-        Returns
-        -------
-            sparse matrix
-        """
         return(self.annihilate[i] + self.create[i])
     
     
     def sigmay(self, i):
-        """
-        A Pauli operator for i-th mode 
-
-        Parameters
-        ----------
-            i: int
-                An index of the mode                         
-                                                  
-        Returns
-        -------
-            sparse matrix
-        """
+ 
         return(1j*(-self.annihilate[i] + self.create[i]))
     
     
     def sigmaz(self, i):
-        """
-        A Pauli operator for i-th mode 
 
-        Parameters
-        ----------
-            i: int
-                An index of the mode                                         
-                            
-        Returns
-        -------
-            sparse matrix
-        """
         return(self.annihilate[i]@self.create[i] - self.create[i]@self.annihilate[i])
   
         
@@ -267,18 +188,7 @@ class fock_space:
       
         
     def occupations(self,i):
-        """
-        Parameters
-        ----------
-            i: int
-                An index of the state
-        
-        Returns
-        -------
-        state: numpy array
-            A numpy array of state for given index
-        """
-        
+     
         if i >= self.dimension :
             print('the number is out of range')
             
@@ -287,18 +197,7 @@ class fock_space:
     
     
     def index(self, state):
-        """
-        Parameters
-        ----------
-            state: tuple, list or array like
-                Fock state of the given Fock space
-        
-        Returns
-        -------
-            i: int
-                An index of the state
-        """
-        
+       
         if len(state) != self.modes:
             print('incorrect size of an array')
         else:
@@ -310,32 +209,7 @@ class fock_space:
         
 class fock_space_kron:
     def __init__(self, f1, f2):
-        """The Kronecker product of two Fock spaces
-        
-        Parameters
-        ----------
-        f1, f2: fock_space class
-                      
-        Returns
-        -------
-        1) self.modes: int
-            A number of modes
-          
-        2) self.dimension : int
-            A number of all posible states
-            
-        3) self.emptyH: scipy sparse matrix
-            A sparse matrix for hamiltonian
-            
-        4) self.eye: scipy sparse matrix
-            An idenitiy operator
-        
-        5) self annihilate: scipy sparse matrix
-            A list of annihilation operators for each mode(from 0 to self.modes-1)
-            
-        6) self create: scipy sparse matrix
-            A list of creation operators for each mode(from 0 to self.modes-1)
-        """
+
         self.f1 = f1
         self.f2 = f2
         
@@ -402,18 +276,6 @@ class fock_space_kron:
                 self.create.append(sparse.kron(f1.eye, f2.create[k-f1.modes]))
             
     def sigmax(self, i):
-        """
-        A Pauli operator for i-th mode 
-
-        Parameters
-        ----------
-            i: int
-                An index of the mode                                          
-                            
-        Returns
-        -------
-            sparse matrix
-        """
         if (i<self.f1.modes):
              return(sparse.kron(self.f1.sigmax(i), self.f2.eye))
         else:
@@ -421,18 +283,7 @@ class fock_space_kron:
 
         
     def sigmay(self, i):
-        """
-        A Pauli operator for i-th mode 
-
-        Parameters
-        ----------
-            i: int
-                An index of the mode                                                 
-                            
-        Returns
-        -------
-            sparse matrix
-        """
+       
         if (i<self.f1.modes):
              return(sparse.kron(self.f1.sigmay(i), self.f2.eye))
         else:
@@ -440,18 +291,7 @@ class fock_space_kron:
 
         
     def sigmaz(self, i):
-        """
-        A Pauli operator for i-th mode 
-
-        Parameters
-        ----------
-            i: int
-                An index of the mode                         
-                                                   
-        Returns
-        -------
-            sparse matrix
-        """
+        
         if (i<self.f1.modes):
              return(sparse.kron(self.f1.sigmaz(i), self.f2.eye))
         else:
@@ -459,17 +299,6 @@ class fock_space_kron:
     
     
     def occupations(self,i):
-        """
-        Parameters
-        ----------
-            i: int
-                An index of the state
-        
-        Returns
-        -------
-        state: numpy array
-            A numpy array of state for given index
-        """
         
         if i >= self.dimension :
             print('the number is out of range')
@@ -481,17 +310,6 @@ class fock_space_kron:
         
         
     def index(self,state):
-        """
-        Parameters
-        ----------
-            state: tuple, list or array like
-                A Fock state from the kronecker product of gives Fock spaces
-        
-        Returns
-        -------
-            i: int
-                An index of the state
-        """
         
         if state.size != self.modes:
             print('incorrect size of an array')
@@ -499,41 +317,8 @@ class fock_space_kron:
             s = list(state)
             return(self.f1.index(state[:self.f1.modes]) * self.f2.dimension + self.f2.index(state[self.f1.modes:]))
         
-def real_time_solver(psi0, dt, tmax, H, Q):
-    """
-    The discription is in developing
-    Parameters
-    ----------
-        psi0: numpy.array
-                Initial state vector
-                
-          dt: number
-               Time step
-                
-        tmax: number
-               Time interval
-                
-           H: 
-            Hamiltonian of the system
-            
-                1) scipy sparse matrix 
-                    When Hamiltonian is constant
-                
-                2) time dependant function, which returns scipy sparse matrix
-                    When Hamiltonian is time related
-                
-           Q: 
-               1) scipy sparse matrix
-                   Observable operators
-               2) a list of scipy sparse matrices
-                   ?
-                
-    Returns
-    -------
-     results: np.array or a list of np.arrays
-            Average of observables
-       
-    """    
+def real_time_solver(psi0, dt, tmax, H, Q = None, final_state = None):
+  
     K = psi0.size
     Nt = int(tmax/dt)+1
     psi = [psi0, psi0]
@@ -543,14 +328,27 @@ def real_time_solver(psi0, dt, tmax, H, Q):
         def H(t):
             return(H_const)
      
-    if type(Q) == list:
+    if Q == None:
+        for i in range(1,Nt):
+            psi_iter_old = psi0
+            psi_iter = np.zeros(K)
+            psi_compare = psi0
+            while (LA.norm(psi_iter-psi_compare)>10**(-6)):
+                s = psi[1]+psi_iter_old
+                psi_iter = psi[1]+ dt/2*(-1j) * H(i*dt+dt/2)@s
+                psi_compare = psi_iter_old
+                psi_iter_old = psi_iter
+            psi[1] = psi_iter
+            
+        return (psi[1])
+        
+    elif type(Q) == list:
         l = len(Q)
         results= np.zeros((l,Nt))
         for j in range(l):
             results[j,0] = abs(np.conj(psi0) @ Q[j] @ psi0)
         
         for i in range(1,Nt):
-            
             psi_iter_old = psi0
             psi_iter = np.zeros(K)
             psi_compare = psi0
@@ -579,8 +377,10 @@ def real_time_solver(psi0, dt, tmax, H, Q):
                 psi_iter_old = psi_iter
             psi[1] = psi_iter
             results[i] = abs(np.conj(psi[1]) @ Q @ psi[1])
-    
-    return(results)
+    if final_state != None :
+        return (results, psi[1])
+    else:
+        return(results)
     
         
 
